import { Card, Checkbox, Col, Row, Select, Spin } from "antd";
import { useEffect, useRef, useState } from "react";
import { Constants, myFetch } from "../../utils";
import { useTranslation } from "react-i18next";
import { Virtuoso } from "react-virtuoso";

export default function LogCard({
  type,
  title,
  customResult,
  logApiAddress = Constants.LOG_API_ADDRESS,
}) {
  const { t } = useTranslation();
  const [checkboxInfoChecked, setCheckboxInfoChecked] = useState(true);
  const [checkboxWarningChecked, setCheckboxWarningChecked] = useState(true);
  const [checkboxErrorChecked, setCheckboxErrorChecked] = useState(true);
  const virtuosoRef = useRef();

  const [loadingSpinner, setLoadingSpinner] = useState(true);
  const [availableDates, setAvailableDates] = useState([]);
  const [selectedDate, setSelectedDate] = useState("");
  const [logs, setLogs] = useState([]);

  const sseEventSource = useRef(null);

  useEffect(() => {
    if (type === "") return;

    // init request to get all available dates
    myFetch(`s/${type}`, "GET", null, {}, 0, logApiAddress)
      .then((data) => {
        let dates = [];

        if (data.length === 0) return;

        dates = data.map((date) => {
          return {
            value: date,
            label: date,
          };
        });

        setAvailableDates(dates);
        setSelectedDate(dates[0].value);

        if (selectedDate === dates[0].value) {
          loadLogs(dates[0].value);
        }
      })
      .catch((errStatus) => {
        if (errStatus === 422) {
          setLoadingSpinner(false);
        }
      });
  }, [type]);

  const loadLogs = (date) => {
    setLoadingSpinner(true);

    if (date === "") return;

    // fetch logs
    myFetch(`s/${type}?d=${date}`, "GET", null, {}, 0, logApiAddress)
      .then((data) => {
        setLoadingSpinner(false);

        // data is undefined when there are no logs for the selected date (e.g. log was deleted)
        if (data === undefined) {
          setLogs([]);
          return;
        }

        setLogs(data);
      })
      .catch((errStatus) => console.log("error", errStatus));
  };

  useEffect(() => loadLogs(selectedDate), [selectedDate]);

  useEffect(() => {
    if (type === "" || selectedDate === "") return;

    const currentDate = new Date();

    const today = `${currentDate.getDate()}-${
      currentDate.getMonth() + 1
    }-${currentDate.getFullYear()}`;

    // only open the sse connection if the selected date is today
    if (selectedDate === today) {
      sseEventSource.current = new EventSource(
        `${logApiAddress}/sse/${type}/${selectedDate}`
      );

      sseEventSource.current.onmessage = (event) => {
        const data = JSON.parse(event.data);

        setLogs((prevLogs) => {
          const newLogs = [...prevLogs];

          data.forEach((log) => {
            newLogs.push(log);
          });

          return newLogs;
        });
      };

      sseEventSource.current.onerror = (event) =>
        console.log("sse error", event);
      /*
      sseEventSource.current.onopen = (event) => console.log("sse open", event);

      sseEventSource.current.onclose = (event) =>
        console.log("sse close", event); */

      return () => sseEventSource.current.close();
    }
  }, [type, selectedDate]);

  const filteredLogs = logs.filter((log) => {
    const logType = log.charAt(0);

    // some other logs which have no type defined
    if (logType !== "I" && logType !== "W" && logType !== "E") {
      return true;
    }

    return (
      (checkboxInfoChecked && logType === "I") ||
      (checkboxWarningChecked && logType === "W") ||
      (checkboxErrorChecked && logType === "E")
    );
  });

  return (
    <Card
      title={
        <Row>
          <Col xs={24} xl={12}>
            {title}
          </Col>

          <Col xs={24} xl={12}>
            <Row xs={24} justify="end" align="middle">
              <Col>
                <Checkbox
                  disabled={customResult !== undefined}
                  checked={checkboxInfoChecked}
                  onChange={(e) => setCheckboxInfoChecked(e.target.checked)}
                >
                  {t("logCard.card.checkbox.info")}
                </Checkbox>
              </Col>
              <Col>
                <Checkbox
                  disabled={customResult !== undefined}
                  checked={checkboxWarningChecked}
                  onChange={(e) => setCheckboxWarningChecked(e.target.checked)}
                >
                  {t("logCard.card.checkbox.warning")}
                </Checkbox>
              </Col>
              <Col>
                <Checkbox
                  disabled={customResult !== undefined}
                  checked={checkboxErrorChecked}
                  onChange={(e) => setCheckboxErrorChecked(e.target.checked)}
                >
                  {t("logCard.card.checkbox.error")}
                </Checkbox>
              </Col>

              <Col>
                <Select
                  disabled={customResult !== undefined}
                  style={{ width: 120 }}
                  options={availableDates}
                  value={selectedDate}
                  onChange={(value) => setSelectedDate(value)}
                />
              </Col>
            </Row>
          </Col>
        </Row>
      }
      style={{
        height: "90vh",
        overflow: "hidden",
        overflowWrap: "break-word",
        width: "100%",
      }}
    >
      {customResult === undefined ? (
        <>
          {!loadingSpinner && type !== "" ? (
            <div style={{ height: "80vh", paddingRight: 6 }}>
              <Virtuoso
                ref={virtuosoRef}
                initialTopMostItemIndex={logs.length - 1}
                data={filteredLogs}
                itemContent={(_, log) => {
                  const logType = log.charAt(0);
                  let style = {};
                  let color = "";

                  if (checkboxInfoChecked && logType === "I") {
                    color = "#44bd32";
                  } else if (checkboxWarningChecked && logType === "W") {
                    color = "#e67e22";
                  } else if (checkboxErrorChecked && logType === "E") {
                    color = "#e74c3c";
                  } else {
                    color = "#2980b9"; // no match
                  }

                  style = { padding: "1px", color, whiteSpace: "pre-line" };

                  return <span style={style}>{log}</span>;
                }}
                followOutput={true}
              />
            </div>
          ) : (
            <div
              style={{
                display: "flex",
                justifyContent: "center",
                alignItems: "center",
                height: "80vh",
              }}
            >
              <Spin size="large"></Spin>
            </div>
          )}
        </>
      ) : (
        <div
          style={{
            display: "flex",
            justifyContent: "center",
            alignItems: "center",
            height: "80vh",
          }}
        >
          {customResult}
        </div>
      )}
    </Card>
  );
}
